// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_EXT_PYTHON_INCLUDED
#define BARETORRENT_EXT_PYTHON_INCLUDED

#include "../path.h"

namespace baretorrent
{
namespace ext
{

// scoped lock for the gil
//
class python_lock : boost::noncopyable
{
public:
	python_lock();
	~python_lock();

private:
	PyGILState_STATE state_;
};

// initializes the python library
//
bool init_python(const std::vector<utf8_path> paths);

// cleans up the python library
//
void term_python();

// whether python was initialized correctly; this prevents from using
// extensions
//
bool python_initialized();


// extracts a string from the given object and returns it as utf8
//
utf8_string extract_string(bpl::object o);

// extracts the bytes object as a string
//
bool extract_bytes(bpl::object o, utf8_string& out);

// returns the last error as a string
//
utf8_string python_error();

// implodes 'ep' and calls the other set_ext_path()
//
void set_ext_path(const std::vector<utf8_path>& ep, bool module_path);

// adds the given string to the path; if 'module_path' is true, this
// path is for python's modules, not the extensions
//
void set_ext_path(const utf8_string& s, bool module_path);


// remembers the current exception in the constructor, sets it back
// in the destructor; this is used by various utility classes that might
// be cleaning up, which would overwrite the original exception
// 
class error_restorer : boost::noncopyable
{
public:
	// remembers the current exception
	//
	error_restorer();

	// resets the old exception
	//
	~error_restorer();

private:
	PyObject* type_;
	PyObject* value_;
	PyObject* tb_;
};

// sets the python path in the constructor, resets it in the destructor;
// this is used to add the extension's path so it can be found with
// import
//
class scoped_ext_path : boost::noncopyable
{
public:
	// appends the given string to the python path
	//
	scoped_ext_path(const std::vector<utf8_path>& ps);

	// appends the given paths to the python path
	//
	scoped_ext_path(const utf8_string& p);

	// resets the old path
	//
	~scoped_ext_path();

private:
	utf8_string old_;
};

// returns whether the object has the given attribute
//
bool has(bpl::object o, const utf8_string& name);

}	// namespace ext
}	// namespace baretorrent

#endif	// BARETORRENT_EXT_PYTHON_INCLUDED
